//
//  Cross Track for Java
//  (c)1997 May 17 Programmed by Bv.
//

import java.lang.*;
import java.awt.*;
import java.io.*;

public class Screen extends Canvas
  {
  final static Color BACK_COLOR  = new Color( 32, 32,255);
  final static Color LINE_COLOR  = new Color( 80, 80,255);
  final static Color BLOCK_COLOR = new Color(255, 32, 32);
  final static Color WALL_COLOR  = new Color(160,160,160);
  final static Color PAD_COLOR   = new Color( 32,255, 32);
  final static Color TEXT_COLOR  = new Color(255,255,255);
  Image image = null;
  Blocks blocks = null;
  
  public Screen(Blocks blocks)
    {
    super();
    this.blocks = blocks;
    resize(256,256);
    }
  
  public void update(Graphics g){ paint(g); }
  
  public void paint(Graphics g)
    {
    if(image != null)
      g.drawImage(image,0,0,this);
    }
  
  public void drawAll()
    {
    if(image == null)
      image = createImage(256,256);
    Graphics img_g = image.getGraphics();
    drawBackground(img_g);
    drawBlocks(img_g);
    img_g.dispose();
    Graphics win_g = getGraphics();
    win_g.drawImage(image,0,0,this);
    }
  
  public void drawGameOver()
    {
    if(image == null)
      image = createImage(256,256);
    Graphics img_g = image.getGraphics();
    drawBackground(img_g);
    drawBlocks(img_g);
    Font font = new Font("Helvetica", Font.BOLD, 24);
    img_g.setFont(font);
    img_g.setColor(TEXT_COLOR);
    FontMetrics metrics = img_g.getFontMetrics();
    String score = "SCORE "+blocks.score();
    int x1 = (256 - metrics.stringWidth(score))/2;
    int x2 = (256 - metrics.stringWidth("GAME OVER"))/2;
    img_g.drawString(score, x1, 96);
    img_g.drawString("GAME OVER", x2, 128);
    img_g.dispose();
    Graphics win_g = getGraphics();
    win_g.drawImage(image,0,0,this);
    }
  
  public void drawComplete(int stagelevel)
    {
    if(image == null)
      image = createImage(256,256);
    Graphics img_g = image.getGraphics();
    drawBackground(img_g);
    drawBlocks(img_g);
    Font font = new Font("Helvetica", Font.BOLD, 24);
    img_g.setFont(font);
    img_g.setColor(TEXT_COLOR);
    FontMetrics metrics = img_g.getFontMetrics();
    String stage = "STAGE " + stagelevel;
    int x1 = (256 - metrics.stringWidth(stage))/2;
    int x2 = (256 - metrics.stringWidth("COMPLETED"))/2;
    img_g.drawString(stage, x1, 96);
    img_g.drawString("COMPLETED", x2, 128);
    img_g.dispose();
    Graphics win_g = getGraphics();
    win_g.drawImage(image,0,0,this);
    }
  
  void drawBackground(Graphics g)
    {
    g.setColor(BACK_COLOR);
    g.fillRect(0,0,256,256);
    int ox = 32 - blocks.X()/4;
    int oy = 32 - blocks.Y()/4;
    g.setColor(LINE_COLOR);
    for(int i=-32; i<352; i+=32)
      {
      g.drawLine(i+ox,oy,i+ox,352+oy);
      g.drawLine(ox,i+oy,352+ox,i+oy);
      }
    }
  
  void drawBlocks(Graphics g)
    {
    int ox = 128 - blocks.X();
    int oy = 128 - blocks.Y();
    for(int i=0; i<16; i++)
      for(int j=0; j<16; j++)
        switch(blocks.block(j,i))
          {
        case Blocks.BLOCK:
          g.setColor(BLOCK_COLOR);
          g.fillRect(j*16+ox,i*16+oy,15,15);
          break;
        case Blocks.WALL:
          g.setColor(WALL_COLOR);
          g.fillRect(j*16+ox,i*16+oy,15,15);
          break;
          }
    g.setColor(PAD_COLOR);
    g.fillOval(124,124,8,8);
    }
  }