//
//  Cross Track for Java
//  (c)1997 May 17 Programmed by Bv.
//

import java.lang.*;
import java.awt.*;
import java.io.*;

public class Blocks extends Object
  {
  public final static byte SPACE = (byte)0;
  public final static byte BLOCK = (byte)1;
  public final static byte WALL  = (byte)2;
  public final static int CONTINUE = 0;
  public final static int CLEAR    = 1;
  public final static int GAMEOVER = 2;
  int x, y, vx, vy;
  int score = 0;
  int nblocks;
  byte[] blocks;
  
  public void loadStage(int stage)
    {  
    blocks = new byte[256];
    nblocks = 0;
    vx=4; vy=-4;
    for(int i=0; i<16; i++)
      for(int j=0; j<16; j++)
        switch(BlockData.get(stage,j,i))
          {
        case '.': blocks[j+i*16]=SPACE; break;
        case 'o': blocks[j+i*16]=BLOCK; nblocks++; break;
        case 'O': blocks[j+i*16]= WALL; break;
        case '*': x=j*16+8; y=i*16+8; break;
          }
    }
  
  public byte block(int j, int i)
    {
    if(blocks == null) return SPACE;
    int k = j+i*16;
    if(k<0 || k>255) return SPACE;
    return blocks[k];
    }
  
  public synchronized boolean destroy(int j, int i)
    {
    if(blocks[j+i*16] == BLOCK)
      {
      blocks[j+i*16] = SPACE;
      score += 1;
      if(--nblocks <= 0) return true;
      }
    return false;
    }
  
  public synchronized boolean move()
    {
    x+=vx; y+=vy;
    if(x>255) x=0; else if(x<0) x=255;
    if(y<0 || y>255) return true;
    return false;
    }
  
  public int X(){ return x; }
  public int Y(){ return y; }
  public int edgeX(){ return (vx>0)? x+4: x-4; }
  public int edgeY(){ return (vy>0)? y+4: y-4; }
  public synchronized void reflectX(){ vx=-vx; }
  public synchronized void reflectY(){ vy=-vy; }
  public int score(){ return score; }
  public void clearScore(){ score = 0; }
  public boolean clear(){ return (nblocks<=0); }
  }

class BlockData extends Object
  {
  final static String block_data[][] = {
    
    {	"..OoOOOOOOOOoO..",	//STAGE 1
	"..O..........O..",
	"..Oooo....oooO..",
	"..Oooo....oooO..",
	"..Oooo....oooO..",
	"..Oooo....oooO..",
	"..O..........O..",
	"..O..........O..",
	"..O..........O..",
	"..O..........O..",
	"..O..........O..",
	"..O..........O..",
	"..O.....*....O..",
	"..O..........O..",
	"..O..........O..",
	"..O...OOOO...O.."	},
	
    {	".......OO.......",	//STAGE 2
	".....O....O.....",
	"...O........O...",
	"..O..........O..",
	"..O.o......o.O..",
	"..O.o......o.O..",
	"..O.oOOOOOOo.O..",
	"..O.oooooooo.O..",
	"..O.o......o.O..",
	"..O.o......o.O..",
	"..O.o......o.O..",
	"..O.o......o.O..",
	"..O.....*....O..",
	"...O........O...",
	".....O....O.....",
	".......OO......."	},
	
    {	"O..............O",	//STAGE 3
	"O..............O",
	"OooooooooooooooO",
	"O..............O",
	"O..............O",
	"O..............O",
	"O..............O",
	"O..............O",
	"O..............O",
	"O..............O",
	"O..............O",
	"O..............O",
	"O.......*......O",
	"O..OO..OO..OO..O",
	"O..............O",
	"O..............O"	},
	
    {	"..OOO......OOO..",	//STAGE 4
	".O............O.",
	"O..............O",
	"O..............O",
	"O.oo........oo.O",
	"O.oo........oo.O",
	"O.oo........oo.O",
	"O.oo........oo.O",
	"O.oo........oo.O",
	"O.oo........oo.O",
	"O.oo........oo.O",
	"O.oo........oo.O",
	"O.......*......O",
	"O..............O",
	".O............O.",
	"..OOO......OOO.."	},
    
    {	"O....OOOOOO....O",	//STAGE 5
	"O..............O",
	"O..............O",
	"O..............O",
	"O..............O",
	"O..............O",
	"OOO...O..O...OOO",
	"O.....O..O.....O",
	"O.....O..O.....O",
	"O.....O..O.....O",
	"O.oo..O..O..oo.O",
	"O.oo..O..O..oo.O",
	"O.oo..O.*O..oo.O",
	"O.oo..OOOO..oo.O",
	"O.oo........oo.O",
	"O.Oo........oO.O"	},

    {	"O.....OOOO.....O",	//STAGE 6
	"Oo............oO",
	"Oo............oO",
	"Oo............oO",
	"Oo............oO",
	"Oo............oO",
	"Oo............oO",
	"Oo............oO",
	"Oo............oO",
	"Oo............oO",
	"Oo............oO",
	"Oo............oO",
	"Oo......*.....oO",
	"Oo............oO",
	"Oo............oO",
	"O.....OOOO.....O"	},
    
    {	"O..............O",	//STAGE 7
	"O..............O",
	"O...OooooooO...O",
	"O..............O",
	"O.O..oooooo..O.O",
	"O.o.o......o.o.O",
	"O.o.o......o.o.O",
	"O.o.o......o.o.O",
	"O.o.o......o.o.O",
	"O.o.o......o.o.O",
	"O.o.o......o.o.O",
	"O.O..oooooo..O.O",
	"O.......*......O",
	"O...OooooooO...O",
	"O..............O",
	"O..............O"	},
    
    {	"OOO..........OOO",	//STAGE 8
	"Oo............oO",
	"Oo............oO",
	"Oo............oO",
	"O.o..........o.O",
	"O.o..........o.O",
	"O.o..........o.O",
	"O.o..........o.O",
	"O.o..........o.O",
	"O.o..........o.O",
	"O.o..........o.O",
	"O.o..........o.O",
	"Oo......*.....oO",
	"Oo............oO",
	"Oo............oO",
	"Oo.....OO.....oO"	}
    };
  
  final static int NUMBER_STAGES = 8;
  final static int ROWS = 16;
  final static int COLUMNS = 16;
  
  public static char get(int stage, int j, int i)
    {
    if(j<0 || j>COLUMNS) return '\0';
    if(i<0 || i>ROWS   ) return '\0';
    if(stage < 0) return '\0';
    int st = (stage - 1) % NUMBER_STAGES;
    return block_data[st][i].charAt(j);
    }
  }